#      BigWindow.py - part of PyGCID
#      Copyright (C) 2000 Erik Forsberg
#      forsberg@lysator.liu.se

#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.

#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.

#      You should have received a copy of the GNU General Public License
#      along with this program, see the file COPYING; if not, write
#      to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#      MA 02139, USA.

from gtk import *
from Window import Window
from common import *
from EditNumberWin import EditNumberWin
import os

class BigWindow(Window):
    def getNumber(self, row):
        return self.numberClist.get_text(row, 0)
    
    def SelectRow(self, clist, row, column, event):
        self.rowSelected = 1
        self.selectedRow = row
        data = self.parent.parent.db.findNumber(self.getNumber(row))
        self.note.delete_text(0, self.note.get_length())
        self.note.insert_text(data['note'])

    def unSelectRow(self, clist, row, column, event):
        self.rowSelected = 0

    def updateList(self):
        self.numberClist.freeze()
        self.numberClist.clear()
        for number in self.parent.parent.db.listNumbers():
            data = self.parent.parent.db.findNumber(number)
            self.numberClist.append([number, data['comment'],
                                     data['lastTime'], data['command']])
        self.numberClist.set_column_width(0, self.numberClist.optimal_column_width(0))
        self.numberClist.set_column_width(1, self.numberClist.optimal_column_width(1))
        self.numberClist.set_column_width(2, self.numberClist.optimal_column_width(2))
        self.numberClist.set_column_width(3, self.numberClist.optimal_column_width(2))
        self.numberClist.thaw()
        
    def editButton_clicked(self, button):
        if(self.rowSelected):
            EditNumberWin(self.getNumber(self.selectedRow),
                          self.parent.parent.db, self.updateList)

    def deleteButton_clicked(self, button):
        if(self.rowSelected):
            self.parent.parent.db.deleteNumber(self.getNumber(self.selectedRow))
            self.numberClist.remove(self.selectedRow)

    def newButton_clicked(self, button):
        EditNumberWin("", self.parent.parent.db, self.updateList)

#    def columnClicked(self, widget, col):
#        self.numberClist.set_sort_column(col)
        
    def DatabaseTab(self):
        mainVbox = GtkVBox()
        mainVbox.show()
        mainVbox.set_spacing(5)
        mainVbox.set_homogeneous(TRUE)
        
        hbox = GtkHBox()
        hbox.show()
        hbox.set_spacing(5)

        buttonBox = GtkVBox()
#        buttonBox.set_usize(40,40)
        buttonBox.show()

        self.numberClist = GtkCList(cols = 4,
                                    titles=["Number", "Comment",
                                            "Last call", "Command"])
        self.updateList()
        self.numberClist.connect("select_row", self.SelectRow)
        self.numberClist.show()

#        self.numberClist.get_column_widget(0).connect("clicked",
#                                                      self.columnClicked,
#                                                      0)
        hbox.pack_start(self.numberClist)

        editButton = GtkButton("Edit")
        editButton.show()
        editButton.connect("clicked", self.editButton_clicked)
        buttonBox.pack_start(editButton)

        deleteButton = GtkButton("Delete")
        deleteButton.show()
        deleteButton.connect("clicked", self.deleteButton_clicked)
        buttonBox.pack_start(deleteButton)

        newButton = GtkButton("New")
        newButton.show()
        newButton.connect("clicked", self.newButton_clicked)
        buttonBox.pack_start(newButton)

        noteFrame = GtkFrame()
        noteFrame.set_label("Note")
        noteFrame.show()
        
        self.note = GtkText()
        self.note.set_editable(FALSE)
        self.note.show()
        noteFrame.add(self.note)

        hbox.pack_start(buttonBox)
        mainVbox.pack_start(hbox)
        mainVbox.pack_start(noteFrame)

        return mainVbox

    def destroy(self, win):
        self.window.destroy()
        self.parent.bigWin = None

    def callNumber(self, widget):
        if(self.rowSelected):
            debug("BigWindow: CallNumber %s" % self.getNumber(self.selectedRow))
            os.system(self.parent.parent.callProgram + \
                      " %s" % self.getNumber(self.selectedRow))

    def __init__(self, parent):
        self.rowSelected = 0
        Window.__init__(self, "PyGCID")
        self.parent = parent
        debug("BigWindow:__init__")
        self.window.connect("destroy", self.destroy)
        self.window.set_border_width(2)
        mainVbox = GtkVBox()
        mainVbox.set_spacing(5)

        noteBook = GtkNotebook()
        databaseTab = self.DatabaseTab()

        noteTabDatabase = GtkLabel("Database")
        noteTabDatabase.show()
        noteBook.append_page(databaseTab, noteTabDatabase)
        noteBook.show()

        mainVbox.pack_start(noteBook)

        buttonHbox = GtkHBox()
        buttonHbox.show()
        closeButton = GtkButton("Close")
        closeButton.show()
        closeButton.connect("clicked", self.destroy)
        buttonHbox.pack_start(closeButton)

        if(None != self.parent.parent.callProgram):
            callButton = GtkButton("Call Number")
            callButton.show()
            callButton.connect("clicked", self.callNumber)
            buttonHbox.pack_start(callButton)
        mainVbox.pack_start(buttonHbox)
        mainVbox.show()
        self.window.add(mainVbox)
        self.window.show()










