#      CallerID.py - part of PyGCID
#      Copyright (C) 2000 Erik Forsberg
#      forsberg@lysator.liu.se

#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.

#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.

#      You should have received a copy of the GNU General Public License
#      along with this program, see the file COPYING; if not, write
#      to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#      MA 02139, USA.

import os
import string
from cpopen import killable_popen
from NumberWin import NumberWin
from MainWin import MainWin
from FileDataBase import FileDataBase
from common import *
from gtk import input_add, mainloop, input_remove
import signal
import sys



class CallerID:
    def mainQuit(self):
        debug("CallerID:mainQuit")
        self.db.deInit()
        self.closeProgram() #FIXME!

    def SigHandler(self, signal, frame):
        debug("Sighandler got signal %d" % signal)
        self.MainWin.window.destroy()

    def __init__(self):
        cidProgram = None
        callProgram = None
        self.cfgPath = os.path.join(os.environ['HOME'], ".cid")
        dataBase = os.path.join(self.cfgPath, "phoneDB")
        if(not os.path.exists(self.cfgPath)):
            prevUmask = os.umask(0)
            os.mkdir(self.cfgPath, 0700)
            os.umask(prevUmask)
        cfgFile = os.path.join(self.cfgPath, "config")
        local = locals()
        if(os.path.exists(cfgFile)):
            debug("Config file %s exists" % cfgFile)
            fp = open(cfgFile, "r")
            exec(fp.read())
            fp.close()
        self.progFd = None
        self.cidProgram = cidProgram
        self.callProgram = callProgram
        self.database = dataBase
        self.inputTag = None
        self.mainWin = MainWin(self)
        self.numberWin = None
        self.db = FileDataBase(self.cfgPath) # FIXME!

    def closeProgram(self):
        if(None != self.progFd):
            print "Trying to kill program"
            try:
                self.progFd.kill(9)
                self.progFd.close()
            except OSError:
                print "External program already dead. Good."
        self.progFd = None

    def openProgram(self):
        debug("openProgram")
        self.closeProgram()
        if(None != self.cidProgram):
            self.progFd = killable_popen(self.cidProgram)
        else:
            debug("No external command for retrieving numbers defined!!")

    def incomingCB(self, one, two):
        text = self.progFd.readline()
        string.strip(text)
        f = string.find(text, "\n")
        if(-1 != f):
            text = text[0:f]
        debug("Incoming call, text: %s" % text)
        numberData = self.db.findNumber(text)
        if(None != numberData):
            debug("Number already in database")
            self.db.updateTime(text, numberData)
        else:
            debug("CallerID:Adding number to database")
            self.db.addNumber(text)
            numberData = self.db.findNumber(text)
            if(None == numberData):
                raise "Oops! Database update failed!"
        self.progFd.close()
        self.deregisterProgramFd()
        self.openProgram()
        self.registerProgramFD()
        if(None == self.numberWin):
            self.numberWin = NumberWin(text, self)
        else:
            self.numberWin.addNumber(text)
        if(None != self.mainWin.bigWin):
            self.mainWin.bigWin.updateList()
        data = self.db.findNumber(text)
        if(data['command'] != ''):
            cmd = string.replace(data['command'], "%s", text)
            os.system(cmd)
            

    def deregisterProgramFd(self):
        if(None != self.inputTag):
            input_remove(self.inputTag)

    def registerProgramFD(self):
        debug("CallerID:registerProgramFD")
        if(None != self.progFd and None != self.progFd.file):
            self.inputTag = input_add(self.progFd.file, 1, self.incomingCB)
                                 
    def main(self):
        self.openProgram()
        self.registerProgramFD()
        mainloop()
        debug("Executing mainQuit")
        self.mainQuit()
