#      cpopen.py - part of PyGCID
#      Copyright (C) 2000 Per Cederqvist
#      ceder@lysator.liu.se
#      $Id: cpopen.py,v 1.2 2000/09/22 18:42:32 erik Exp $

#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.

#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.

#      You should have received a copy of the GNU General Public License
#      along with this program, see the file COPYING; if not, write
#      to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#      MA 02139, USA.
import os
import sys

class killable_popen:
    def __init__(self, command, mode = "r", bufsize = -1):
        if mode not in ["r", "w"]:
            raise ValueError('the mode argument must be "r" or "w"')
        (rd, wr) = os.pipe()
        if mode == "r":
            (parent_fd, child_fd) = (rd, wr)
        else:
            (child_fd, parent_fd) = (rd, wr)
        sys.stdin.flush()
        sys.stdout.flush()
        sys.stderr.flush()
        child = os.fork()
        if child == 0:
            # The child
            os.close(parent_fd)
            if mode == "r":
                wanted_fd = 1
            else:
                wanted_fd = 0
            if child_fd != wanted_fd:
                os.dup2(child_fd, wanted_fd)
                os.close(child_fd)
            os.execvp(command[0], command)

        # The parent
        os.close(child_fd)
        self.__dict__["file"] = os.fdopen(parent_fd, mode, bufsize)
        self.__dict__["pid"] = child

    def close(self):
        self.file.close()
        (pid, ret) = os.waitpid(self.pid, 0)
        if ret == 0:
            return None
        else:
            return ret

    def kill(self, sig):
        os.kill(self.pid, sig)

    # Emulate a file object, so that read(), readline(), write(), et c works.
    def __getattr__(self, name):
        return getattr(self.file, name)

    def __setattr__(self, name, value):
        return self.file.__setattr__(name)

# Test/demo code below.
if __name__ == '__main__':
    import time
    import signal

    p = killable_popen(["/bin/echo", "hej"], "r")
    print "hej =>", repr(p.read())
    print "None =>", p.close()

    p = killable_popen(["/bin/echo", "hej", "hopp"], "r")
    print "hej hopp =>", repr(p.read())
    print "None =>", p.close()
    
    p = killable_popen(["/bin/sleep", "3600"], "r")
    time.sleep(1)
    p.kill(signal.SIGINT)
    print "<nothing> =>", repr(p.read())
    print "exit-status =>", p.close()

    p = killable_popen(["/bin/od", "-c"], "w")
    p.write("hej")
    print "None =>", p.close()

    p = killable_popen(["/bin/od", "-c"], "w")
    p.write("hopphopphopphopphopphopphopphopp")
    p.flush()
    time.sleep(1)
    p.kill(signal.SIGQUIT)
    print "exit-status =>", p.close()
